#!/usr/bin/perl
use strict;
use warnings;
use Text::CSV qw( csv );
use Getopt::Long;

my $directory = "botti-massa";
my $file = "botti-massa.tex";
my $standalone;
GetOptions("directory=s"  => \$directory,
           "file=s"       => \$file,
           "standalone" => \$standalone)
    or die("Error in command line arguments");

open(my $fh, '>', $file);

# Table header
if ($standalone) {
print $fh <<EOF;
\\documentclass{standalone}
\\usepackage{booktabs}
\\begin{document}
EOF
}

print $fh <<EOF;
\\begin{tabular}{ccccccccccccc}
  \\toprule
  \$h\$
  & \$k\$
  & System size
  & \$\\| \\underline{u}_h - \\underline{I}_{U,h} u \\|_{1,h}\$ & OCV
  & \$\\| \\nabla_h (r_h \\underline{u}_h - u) \\\|_{L^2(\\Omega)^{d\\times d}}\$ & OCV
  & \$\\| u_h - I_{U,h} u \\|_{L^2(\\Omega)^d}\$ & OCV
  & \$\\| r_h \\underline{u}_h - u \\\|_{L^2(\\Omega)^d}\$ & OCV
  & \$\\| p_h - I_{P,h} p \\|_{L^2(\\Omega)}\$ & OCV
  \\\\
  \\midrule
EOF

# Table body
foreach my $nu ("1.e0", "1.e-3", "1.e-6", "1.e-9") {
    printf $fh "\\multicolumn{13}{c}{\$ \\nu = %.g \$} \\\\ \n", $nu;
    print $fh "\\midrule\n";
    
    foreach my $k (0..2) {
        
        my $input_file = "$directory/tri_k$k\_nu$nu/data.dat";
        my $data = csv(in => $input_file, headers => "auto");        
        foreach (@$data) {
            print $fh "$_->{'MeshSize'} & $k & $_->{'SizeSystem'}".
                "& $_->{'H1VelocityComponentNormError'} & $_->{'H1VelocityComponentNormErrorRate'}".
                "& $_->{'H1VelocityPotentialError'} & $_->{'H1VelocityPotentialErrorRate'}".
                "& $_->{'L2VelocityComponentNormError'} & $_->{'L2VelocityComponentNormErrorRate'}".
                "& $_->{'L2VelocityPotentialError'} & $_->{'L2VelocityPotentialErrorRate'}".
                "& $_->{'L2PressureComponentNormError'} & $_->{'L2PressureComponentNormErrorRate'}".
                "\\\\ \n";
        }
        print $fh ( ($nu == "1.e-9" && $k == 2) ? "\\bottomrule\n" : "\\midrule\n" );
    }
}

# Table footer
print $fh "\\end{tabular}";
if ($standalone) {
    print $fh "\n\\end{document}";
}
